/* eslint-env browser */
import * as React from 'react';
import Prefixer from 'inline-style-prefixer';
import { TransitionContext } from 'react-transition-context';
var bpfrpt_proptype_ViewProps = {
  "index": PropTypes.number.isRequired,
  "active": PropTypes.bool.isRequired,
  "transitionState": PropTypes.oneOf(["in", "out", "entering", "leaving"]).isRequired
};
var bpfrpt_proptype_DefaultProps = {
  "animateHeight": PropTypes.bool.isRequired,
  "keepViewsMounted": PropTypes.bool.isRequired,
  "transitionDuration": PropTypes.number.isRequired,
  "transitionTimingFunction": PropTypes.string.isRequired,
  "prefixer": function () {
    return (typeof Prefixer === "function" ? PropTypes.instanceOf(Prefixer).isRequired : PropTypes.any.isRequired).apply(this, arguments);
  },
  "style": PropTypes.object.isRequired,
  "viewportStyle": PropTypes.object.isRequired,
  "rtl": PropTypes.bool.isRequired,
  "spacing": PropTypes.number.isRequired
};
var bpfrpt_proptype_Props = {
  "activeView": PropTypes.number.isRequired,
  "numViews": PropTypes.number.isRequired,
  "renderView": PropTypes.func.isRequired,
  "keepViewsMounted": PropTypes.bool,
  "animateHeight": PropTypes.bool,
  "transitionDuration": PropTypes.number,
  "transitionTimingFunction": PropTypes.string,
  "onSlideTransitionEnd": PropTypes.func,
  "prefixer": function () {
    return (typeof Prefixer === "function" ? PropTypes.instanceOf(Prefixer) : PropTypes.any).apply(this, arguments);
  },
  "fillParent": PropTypes.bool,
  "className": PropTypes.string,
  "style": PropTypes.object,
  "viewportClassName": PropTypes.string,
  "viewportStyle": PropTypes.object,
  "viewStyle": PropTypes.object,
  "innerViewWrapperStyle": PropTypes.object,
  "rootRef": PropTypes.func,
  "viewportRef": PropTypes.func,
  "rtl": PropTypes.bool,
  "spacing": PropTypes.number
};
var bpfrpt_proptype_State = {
  "height": PropTypes.number,
  "transitioning": PropTypes.bool.isRequired,
  "activeView": PropTypes.number.isRequired,
  "numViews": PropTypes.number.isRequired,
  "prevActiveView": PropTypes.number
};
const fillStyle = {
  position: 'absolute',
  top: 0,
  left: 0,
  right: 0,
  bottom: 0
};
const baseViewStyle = {
  display: 'inline-block',
  verticalAlign: 'top',
  whiteSpace: 'normal',
  width: '100%'
};
export const defaultProps = {
  animateHeight: true,
  transitionDuration: 500,
  transitionTimingFunction: 'ease',
  keepViewsMounted: false,
  prefixer: new Prefixer(),
  style: {},
  viewportStyle: {},
  rtl: false,
  spacing: 1
};
function applyDefaults(props) {
  const result = {
    ...props
  };
  for (const key in defaultProps) {
    if (Object.prototype.hasOwnProperty.call(defaultProps, key) && props[key] == null) {
      result[key] = defaultProps[key];
    }
  }
  return result;
}
export default class ViewSlider extends React.Component {
  state = {
    height: undefined,
    transitioning: false,
    activeView: this.props.activeView,
    numViews: this.props.numViews,
    // this is used to determine the correct transitionState for the previous active view.
    prevActiveView: null
  };
  views = [];
  timeouts = {};
  lastProps = this.props;
  getDefaultedProps = () => {
    if (this.lastProps !== this.props || !this.lastDefaultedProps) {
      this.lastProps = this.props;
      this.lastDefaultedProps = applyDefaults(this.props);
    }
    return this.lastDefaultedProps;
  };
  measureHeight = node => {
    if (!node) return null;
    return node.clientHeight;
  };
  setTimeout(name, callback, delay) {
    if (this.timeouts[name]) clearTimeout(this.timeouts[name]);
    this.timeouts[name] = setTimeout(callback, delay);
  }
  componentDidUpdate() {
    const {
      activeView,
      transitionDuration,
      keepViewsMounted
    } = this.getDefaultedProps();
    let newState;
    if (activeView !== this.state.activeView && this.state.height === undefined) {
      if (keepViewsMounted) {
        // scroll all views except the current back to the top
        for (let i = 0; i < this.views.length; i++) {
          if (i === this.state.activeView) continue;
          if (this.views[i]) this.views[i].scrollTop = 0;
        }
      }
      // phase 1: set current height
      newState = {
        height: this.measureHeight(this.views[this.state.activeView])
      };
    } else if (this.state.height !== undefined && !this.state.transitioning) {
      // phase 2: enable transitions
      newState = {
        transitioning: true
      };
    } else if (activeView !== this.state.activeView) {
      // phase 3: change height/activeView
      newState = {
        activeView,
        numViews: Math.max(this.state.numViews, activeView + 1),
        prevActiveView: this.state.activeView,
        height: this.measureHeight(this.views[activeView])
      };
    }
    const finalNewState = newState;
    if (!finalNewState) return;
    this.setState(finalNewState, () => {
      if (finalNewState.activeView != null) {
        this.setTimeout('onTransitionEnd', this.onTransitionEnd, transitionDuration);
      }
    });
  }
  onTransitionEnd = event => {
    // ignore transitionend events from deeper components
    if (event && event.target !== this.viewport) return;
    // phase 0: unset height and disable transitions
    this.setState({
      height: undefined,
      numViews: this.props.numViews,
      prevActiveView: null,
      transitioning: false
    }, () => {
      const {
        onSlideTransitionEnd
      } = this.props;
      if (onSlideTransitionEnd) onSlideTransitionEnd();
    });
  };
  componentWillUnmount() {
    for (let name in this.timeouts) clearTimeout(this.timeouts[name]);
  }
  getTransitionState = childIndex => {
    const {
      activeView,
      prevActiveView
    } = this.state;
    if (prevActiveView == null) return childIndex === activeView ? 'in' : 'out';
    if (childIndex === activeView) return 'entering';
    if (childIndex === prevActiveView) return 'leaving';
    return 'out';
  };
  renderView = index => {
    const {
      fillParent,
      prefixer,
      keepViewsMounted,
      spacing,
      rtl,
      viewStyle,
      innerViewWrapperStyle
    } = this.getDefaultedProps();
    const {
      activeView,
      transitioning
    } = this.state;
    const style = {
      ...baseViewStyle,
      ...viewStyle
    };
    if (fillParent) {
      Object.assign(style, fillStyle);
      style.overflow = 'auto';
      if (rtl) style.right = `${index * spacing * 100}%`;else style.left = `${index * spacing * 100}%`;
    } else if (index > 0) {
      if (rtl) style.marginRight = `${(spacing - 1) * 100}%`;else style.marginLeft = `${(spacing - 1) * 100}%`;
    }

    // when not transitioning, render empty placeholder divs before the active view to push it into the right
    // horizontal position
    if (!transitioning && activeView !== index && !keepViewsMounted) {
      return /*#__PURE__*/React.createElement("div", {
        key: index,
        style: prefixer.prefix(style)
      });
    }
    return /*#__PURE__*/React.createElement("div", {
      key: index,
      style: prefixer.prefix(style),
      ref: c => this.views[index] = c
    }, /*#__PURE__*/React.createElement("div", {
      style: prefixer.prefix({
        width: '100%',
        ...innerViewWrapperStyle
      })
    }, /*#__PURE__*/React.createElement(TransitionContext, {
      state: this.getTransitionState(index)
    }, this.props.renderView({
      index,
      active: index === activeView,
      transitionState: this.getTransitionState(index)
    }))));
  };
  animateHeight = () => {
    const {
      animateHeight,
      fillParent,
      keepViewsMounted
    } = this.getDefaultedProps();
    return animateHeight && !fillParent && !keepViewsMounted;
  };
  rootRef = node => {
    this.root = node;
    const {
      rootRef
    } = this.getDefaultedProps();
    if (rootRef) rootRef(node);
  };
  viewportRef = node => {
    this.viewport = node;
    const {
      viewportRef
    } = this.getDefaultedProps();
    if (viewportRef) viewportRef(node);
  };
  render() {
    const {
      style,
      className,
      viewportClassName,
      viewportStyle,
      prefixer,
      fillParent,
      transitionDuration,
      transitionTimingFunction,
      keepViewsMounted,
      rtl,
      spacing
    } = this.getDefaultedProps();
    const animateHeight = this.animateHeight();
    const {
      activeView,
      numViews,
      height,
      transitioning
    } = this.state;
    const finalOuterStyle = {
      transitionProperty: 'height',
      transitionDuration: `${transitionDuration}ms`,
      transitionTimingFunction,
      overflow: 'hidden',
      height: animateHeight && height != null ? height : undefined,
      ...style
    };
    const finalViewportStyle = {
      position: 'relative',
      transform: `translateX(calc(${activeView * spacing * (rtl ? 100 : -100)}% + 0px))`,
      whiteSpace: 'nowrap',
      minHeight: '100%',
      direction: rtl ? 'rtl' : 'ltr',
      transition: transitioning ? `transform ${transitionTimingFunction} ${transitionDuration}ms` : undefined,
      ...viewportStyle
    };
    if (fillParent) {
      Object.assign(finalOuterStyle, fillStyle);
      Object.assign(finalViewportStyle, fillStyle);
    }

    // when not transitioning, render empty placeholder divs before the active view to push it into the right
    // horizontal position
    const views = [];
    for (let i = 0; i < (transitioning || keepViewsMounted ? numViews : activeView + 1); i++) {
      views[i] = this.renderView(i);
    }
    return /*#__PURE__*/React.createElement("div", {
      style: prefixer.prefix(finalOuterStyle),
      className: className,
      ref: this.rootRef
    }, /*#__PURE__*/React.createElement("div", {
      className: viewportClassName,
      style: prefixer.prefix(finalViewportStyle),
      ref: this.viewportRef,
      onTransitionEnd: this.onTransitionEnd
    }, views));
  }
  static propTypes = {
    "activeView": PropTypes.number.isRequired,
    "numViews": PropTypes.number.isRequired,
    "renderView": PropTypes.func.isRequired,
    "keepViewsMounted": PropTypes.bool,
    "animateHeight": PropTypes.bool,
    "transitionDuration": PropTypes.number,
    "transitionTimingFunction": PropTypes.string,
    "onSlideTransitionEnd": PropTypes.func,
    "prefixer": function () {
      return (typeof Prefixer === "function" ? PropTypes.instanceOf(Prefixer) : PropTypes.any).apply(this, arguments);
    },
    "fillParent": PropTypes.bool,
    "className": PropTypes.string,
    "style": PropTypes.object,
    "viewportClassName": PropTypes.string,
    "viewportStyle": PropTypes.object,
    "viewStyle": PropTypes.object,
    "innerViewWrapperStyle": PropTypes.object,
    "rootRef": PropTypes.func,
    "viewportRef": PropTypes.func,
    "rtl": PropTypes.bool,
    "spacing": PropTypes.number
  };
}
import PropTypes from "prop-types";
export { bpfrpt_proptype_ViewProps };
export { bpfrpt_proptype_DefaultProps };
export { bpfrpt_proptype_Props };
export { bpfrpt_proptype_State };